﻿unit UQuickQuickEdit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, System.Rtti,
  Vcl.AppEvnts, Vcl.Controls, Vcl.Forms, FMX.Platform.Win, ToolsAPI, DesignIntf;

procedure Register;

implementation

var
  FAppEvents: TApplicationEvents;

function CreateMethod(Code, Data: Pointer): TMethod;
begin
  Result.Code := Code;
  Result.Data := Data;
end;

procedure AppMessageHandler(Self: TObject; var Msg: TMsg; var Handled: Boolean);
var
  activeControl: TComponent;
  designer: IDesigner;
  selections: IDesignerSelections;

  function GetDesigner: IDesigner;
  var
    module: IOTAModule;
    i: Integer;
    formEditor: INTAFormEditor;
  begin
    Result := nil;
    module := (BorlandIDEServices as IOTAModuleServices).CurrentModule;
    if module = nil then Exit;
    for i := 0 to module.ModuleFileCount-1 do
    begin
      if Supports(module.ModuleFileEditors[i], INTAFormEditor, formEditor) then
        Exit(formEditor.FormDesigner);
    end;
  end;

  procedure ShowQuickEditDlg(Designer: IDesigner; Selections: IDesignerSelections);
  var
    ctx: TRttiContext;
    typ: TRttiType;
    methCreate, methInitFrom: TRttiMethod;
    dlg: Vcl.Forms.TForm;
  begin
    if Designer = nil then Exit;
    if Selections.Count <> 1 then Exit;
    if not (Selections[0] is TComponent) then Exit;

    typ := ctx.FindType('Vcl.Design.QuickEdit.Editor.TQuickEditDlg');
    if typ = nil then Exit;
    methCreate := typ.GetMethod('Create');
    if methCreate = nil then Exit;
    methInitFrom := typ.GetMethod('InitFrom');
    if methInitFrom = nil then Exit;
    dlg := Vcl.Forms.TForm(methCreate.Invoke(TRttiInstanceType(typ).MetaclassType, [Application]).AsObject);
    if dlg = nil then Exit;
    methInitFrom.Invoke(dlg, [Selections[0], TValue.From(Designer)]);
    dlg.Show;
  end;

begin
  if Msg.message <> WM_KEYDOWN then Exit;
  if Msg.wParam <> VK_F2 then Exit;
  if KeyDataToShiftState(Msg.lParam) <> [] then Exit;

  activeControl := FindControl(Msg.hwnd);
  if activeControl = nil then
    activeControl := FindWindow(Msg.hwnd);
  if activeControl = nil then Exit;

  designer := GetDesigner;
  if designer = nil then Exit;
  if designer.Root <> activeControl then Exit;

  selections := CreateSelectionList;
  designer.GetSelections(selections);
  ShowQuickEditDlg(designer, selections);

  Handled := True;
end;

procedure Register;
begin
  FAppEvents := TApplicationEvents.Create(nil);
  FAppEvents.OnMessage := TMessageEvent(CreateMethod(@AppMessageHandler, nil));
end;

procedure Unregister;
begin
  FAppEvents.Free;
end;

initialization
finalization
  Unregister;
end.
